from buildbot.www.plugin import Application
from .api import Api

# create the interface for the setuptools entry point
ep = Application(__name__, "dbconfig")
api = Api(ep)
ep.resource.putChild(b"api", api.app.resource())

__all__ = ['ep']




from klein import Klein
from twisted.internet import defer
from twisted.internet import threads
import json

from buildbot import config

def getDbConfigObjectId(master, name="config"):
    return master.db.state.getObjectId(name, "DbConfig")

class Api(object):
    app = Klein()

    def __init__(self, ep):
        self.ep = ep
        self._cfg = {}

    @app.route("/config", methods=['GET'])
    @defer.inlineCallbacks
    def getConfig(self, request):
        request.setHeader('Content-Type', 'application/json')
        name = request.args.get(b'name',['_'])[0].decode('utf8')
        if name not in self._cfg:
            self._cfg[name] = yield self.loadCfg(name)
        defer.returnValue(json.dumps(self._cfg[name]))

    @app.route("/config", methods=['PUT'])
    @defer.inlineCallbacks
    def putConfig(self, request):
        """I save the config, and run check_config, potencially returning errors"""
        request.setHeader('Content-Type', 'application/json')
        name = request.args.get(b'name',['_'])[0].decode('utf8')
        cfg = json.loads(request.content.read())
        if name not in self._cfg or cfg != self._cfg[name]:
            self._cfg[name] = cfg
            try:
                err = yield self.saveCfg(name,cfg)
            except Exception as e:  # noqa
                err = [repr(e)]
            if err is not None:
                defer.returnValue(json.dumps({'success': False, 'errors': err}))

        yield defer.returnValue(json.dumps({'success': True}))

    @defer.inlineCallbacks
    def saveCfg(self,name,cfg):
        oid = yield getDbConfigObjectId(self.ep.master)
        yield self.ep.master.db.state.setState(oid, name, cfg)

    @defer.inlineCallbacks
    def loadCfg(self,name):
        oid = yield getDbConfigObjectId(self.ep.master)
        try:
            cfg = yield self.ep.master.db.state.getState(oid, name)
            defer.returnValue(cfg)
        except Exception as e:
            defer.returnValue(json.dumps({}))
       



#!/usr/bin/env python

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

try:
    from buildbot_pkg import setup_www_plugin
except ImportError:
    import sys
    print("Please install buildbot_pkg module in order to install that package, or use the pre-build .whl modules available on pypi", file=sys.stderr)
    sys.exit(1)


setup_www_plugin(
    name='buildbot-dbconfig',
    description='Buildbot DbConfig plugin',
    author=u'Alexey Demakov',
    author_email=u'moris.aye.aye@gmail.com',
    license='LGPLv3',
    url='https://github.com/ayaye/buildbot-dbconfig',
    long_description=open('README.rst', 'r').read(),
    packages=['buildbot_dbconfig'],
    package_data={
        '': [
            'VERSION',
            'static/*'
        ]
    },
    entry_points="""
        [buildbot.www]
        dbconfig = buildbot_dbconfig:ep
    """,
    install_requires=[
          'klein',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
)
